//#define VARIANCE_SHADOW_MAPPING

#define MAX_LIGHT 2
#define MAX_CAU_CASTER 2
#define SPOT_ANGLE 2.093
#define SPOT_FALLOFF 1
#define SHADOW_COLOR float4(0.3,0.3,0.3,1.0)
#define SHADOW_BIAS_POINT 0.0001
#define SHADOW_EPSILON_POINT 0.0001

#include "illum.hlsl"

uniform sampler2D tileTexture : register(s0);
uniform sampler2D detailTexture : register(s1);
uniform sampler2D bumpMap : register(s2);
uniform sampler2D PathMap : register(s3);
uniform sampler2D WeightIndexMap : register(s4);
uniform sampler2D WeightMap : register(s5);
uniform samplerCUBE ShadowMap1Point : register(s6);
uniform samplerCUBE ShadowMap2Point : register(s7);

uniform float4x4 WorldViewProj;
uniform float4x4 World;
uniform float4x4 WorldI;

uniform float3 wCamPos;

uniform float4x4 LightView[MAX_LIGHT];
uniform float lightFarPlane[MAX_LIGHT];

uniform float specularity;


struct MORIA_HALL_BASE_VSOUT
{
	float4 hPos : POSITION;
	float4 texCoord   	  : TEXCOORD0;
	float2 bumpTexCoord   : TEXCOORD1;
	float3 wView		  : TEXCOORD2;
	float3 wTangent 	  : TEXCOORD3;
	float3 wBinormal 	  : TEXCOORD4;
	float3 wNormal 		  : TEXCOORD5;
	float3 wPos			  : TEXCOORD6;	
};

MORIA_HALL_BASE_VSOUT MoriaHallBase_VS(float4 position :POSITION,
										float3 normal :NORMAL,
										float2 texCoord1 : TEXCOORD0,
										float2 texCoord2 : TEXCOORD1,
										float3 tangent   : TEXCOORD3,
										uniform float normalCoord)
{
	MORIA_HALL_BASE_VSOUT OUT = (MORIA_HALL_BASE_VSOUT) 0;
	
	OUT.texCoord = float4(texCoord1.xy, texCoord2.xy);	
	if(normalCoord == 0)
		OUT.bumpTexCoord = texCoord1;
	else
		OUT.bumpTexCoord = texCoord2;
		
	OUT.hPos = mul(WorldViewProj, position);
	OUT.wPos = mul(World, position);
	
	OUT.wView = wCamPos - OUT.wPos;
	
	OUT.wTangent = normalize(mul(float4(tangent, 1),WorldI)).xyz;
	OUT.wNormal = normalize(mul(float4(normal, 1),WorldI)).xyz;		
	OUT.wBinormal = cross(OUT.wTangent, OUT.wNormal).xyz;		
		
	return OUT;
}

float4 MoriaHallBase_PS(MORIA_HALL_BASE_VSOUT IN,
						uniform float4 wLightPos,
						uniform float4 lightRange,
						uniform float lightPower,
						uniform float4 lightColor):COLOR0
{	
	float4 Color = 0;
	float3 N = NormalMap(IN.wTangent, IN.wBinormal, IN.wNormal, IN.bumpTexCoord, bumpMap);
	float3 V = normalize(IN.wView);
	// read textures
	float4 diffuseColor = tex2D(tileTexture, IN.texCoord.zw) * tex2D(detailTexture, IN.texCoord.xy);
		
	//light dir
	float3 L = wLightPos.xyz - IN.wPos * wLightPos.w;
	//illumination
	float4 col;
	col = Illumination(N, L, V, lightColor * lightPower, lightRange, diffuseColor, specularity);
	//shadowing
	float3 lightCPos = mul(LightView[0], float4(IN.wPos, 1)).xyz;
	float shadow = shadowPoint(ShadowMap1Point, lightCPos, lightFarPlane[0]);
	Color += col * shadow;	
	
	//do Path Map to gather indirect illumination
	//gather caustics for all casters
	//return diffuseColor;
	return Color;
}