
float4 Illumination(float3 N, float3 L, float3 V, float4 lightColor, float4 lightRange, float4 diffuseColor, float specularity)
{
	// Blinn lighting
	float d = length(L);
	L = normalize(L);
	float3 H = normalize(L + V);
	float4 Lighting = lit(dot(N, L), dot(N, H), specularity);
	
	return   lightColor * (Lighting.y * diffuseColor + Lighting.z) //color
			 / (lightRange.y + d * lightRange.z + d * d * lightRange.w) //attenuation
			 ;
}

float3 NormalMap(float3 tangent, float3 binormal, float3 normal, float2 texCoord, sampler2D normalMap)
{	
	normal = normalize(normal);	
	//return normal;
	float3 mNormal;
	float3 tNormal = tex2D(normalMap, texCoord).rgb;
	if(length(tNormal == 0))	
		mNormal = normal;
    else
    {
    	tangent = normalize(tangent);
		binormal = normalize(binormal);	
		float3x3 TangentToModel = float3x3(tangent, binormal, normal );
		
    	tNormal.xy = (tNormal.xy *2.0) - 1.0;
    	mNormal = normalize( mul( tNormal, TangentToModel ) );
	}
    return mNormal;
}


float shadowPoint(samplerCUBE shadowMap, float3 lightCPos, float lightFarPlane)
{
	  float light = 1;
	  
	  float dist = length(lightCPos) / lightFarPlane;
	  float4 storedDist = texCUBE(shadowMap, float3(lightCPos.xy, -lightCPos.z));
	  dist -= SHADOW_BIAS_POINT;
	  float lit_factor = (dist <= storedDist.r);	
	    	 
	  float M1 = storedDist.r;
	  float M2 = storedDist.g;
	  float v2 = min(max(M2 - M1 * M1, 0.0) +  SHADOW_EPSILON_POINT, 1.0);
	  float m_d = M1 - dist;
	  float pmax = v2 / (v2 + m_d * m_d);
							
	  light = max(lit_factor, pmax);   	
	  	  
	  return (SHADOW_COLOR + (1 - SHADOW_COLOR) * light);
}